### Sparse radiocarbon data confound climate-culture links in late pre-Columbian Amazonia
### Philip Riris
### UCL Institute of Archaeology

require(dplyr)
require(devtools)
install_github("ahb108/rcarbon")
require(rcarbon)

## 1. DATA SETUP

# a. Full dataset

x <- read.csv("full_dataset.csv", header=TRUE)

dataset <- x %>% 
  filter(Phase == "Arauquinoid" | 
           Phase == "Koriabo" | 
           Phase == "Santarm" | 
           Phase == "Marajoara" | 
           Phase =="Guarita" | 
           Phase == "Paredo" |
           Phase == "Mound village" | 
           Phase == "Geoglyph" | 
           Phase == "Zanja" | 
           Phase == "Loma" | 
           Phase == "Xinguano") # Filter out "phases" not used in the analysis

dataset$Phase <- droplevels(dataset$Phase) # Drop unused phase names

dataset$SiteCode <- paste("S",as.numeric(dataset$SiteName),sep="") # Assign unique site identifiers

dtbl <- dataset %>% group_by(Phase) # Setup for KDEs
dtbl <- group_split(dtbl)

# b. Truncated dataset

y <- read.csv("truncated_dataset.csv", header=TRUE) # Load truncated dataset

trunc <- y %>% 
  filter(Phase == "Arauquinoid" | 
           Phase == "Koriabo" | 
           Phase == "Santarm" | 
           Phase == "Marajoara" | 
           Phase =="Guarita" | 
           Phase == "Paredo" |
           Phase == "Mound village" | 
           Phase == "Geoglyph" | 
           Phase == "Zanja" | 
           Phase == "Loma" | 
           Phase == "Xinguano") # Filter out "phases" not used in the analysis

trunc$Phase <- droplevels(trunc$Phase) # Drop unused phase names

trunc$SiteCode <- paste("S",as.numeric(trunc$SiteName),sep="") # Assign unique site identifiers

ttbl <- trunc %>% group_by(Phase) # Setup for KDEs
ttbl <- group_split(ttbl)

## 2. CALIBRATION

timerange <- c(2000, 200)

# a. Full dataset

bins <- binPrep(sites = dataset$SiteCode, ages = dataset$Date, 200) 
calDates <- calibrate(dataset$Date, errors = dataset$Error, normalised = FALSE, calCurves = amazoncurve, ncores=3)

# b. Truncated dataset

Tbins <- binPrep(sites = trunc$SiteCode, ages = trunc$Date, 200) 
calTrunc <- calibrate(trunc$Date, errors = trunc$Error, normalised = FALSE, calCurves = amazoncurve, ncores=3)

## 3. ANALYSIS

# 3.1 Summed probability distributions

# a. Full dataset

sump <- spd(calDates, timeRange = timerange, bins = bins, datenormalised = FALSE, spdnormalised = TRUE, runm = 100)

res.spd <- list() # Store results in list

for(i in 1:11){
  tcal <- calibrate(dtbl[[i]]$Date, errors = dtbl[[i]]$Error, normalised = FALSE, calCurves = amazoncurve, ncores=3)
  tbins <- binPrep(sites = dtbl[[i]]$SiteCode, ages = dtbl[[i]]$Date, 200)
  res.spd[[i]] <- spd(tcal, runm=100, timeRange = timerange, bins = tbins, datenormalised = FALSE, spdnormalised = TRUE)
}

# b. Truncated dataset

sumt <- spd(calTrunc, timeRange = timerange, bins = Tbins, datenormalised = FALSE, spdnormalised = TRUE, runm = 100)

t.spd <- list() # Store results in list

for(i in 1:11){
  tcal <- calibrate(ttbl[[i]]$Date, errors = ttbl[[i]]$Error, normalised = FALSE, calCurves = amazoncurve, ncores=3)
  tbins <- binPrep(sites = ttbl[[i]]$SiteCode, ages = ttbl[[i]]$Date, 200)
  t.spd[[i]] <- spd(tcal, runm=100, timeRange = timerange, bins = tbins, datenormalised = FALSE, spdnormalised = TRUE)
}

# 3.2 Permutation testing

ptestn <- permTest(calDates, marks = as.character(dataset$Phase), timeRange = timerange, nsim = 10000,
                   datenormalised = FALSE, spdnormalised = TRUE, bins = bins, runm=100)

#NOT RUN - Print p-values

#round(ptestn$pValueList, 4)

# * = significant to p < 0.05

#Koriabo*  Arauquinoid     Santarm*     Marajoara       Guarita       Paredo      Geoglyph* Mound village       Zanja 
#0.0043        0.3447        0.0093        0.1022        0.0783        0.2754        0.0001        1.0000        0.2183 
#Loma*       Xinguano 
#0.0333        0.4375

# 3.3 Composite Kernel Density Estimates (all)

res.kde <- list() # Store results in (a very large) list - user beware

for(i in 1:11){
  tcal <- calibrate(dtbl[[i]]$Date, errors = dtbl[[i]]$Error, normalised = FALSE, calCurves = amazoncurve, ncores=3)
  tbins <- binPrep(sites = dtbl[[i]]$SiteCode, ages = dtbl[[i]]$Date, 200)
  tsamp <- sampleDates(tcal, bins=tbins, nsim=10000, boot=TRUE)
  res.kde[[i]] <- ckde(tsamp, timeRange = timerange, normalised=FALSE, bw=25)
}

## 4. PLOTTING

# Figure 1 - Summed probability distributions

par(mfrow=c(2,6))
par(mar=c(2,2,2,2))

plot(sump, calendar = "BCAD")
plot(sumt, calendar = "BCAD", type="simple", ylim=c(0,0.04), add=TRUE, lty=2 )
mtext("Full dataset", font = 2)
text(labels = "dates = 337\nbins = 116\nsites = 57", y=0.00115, x=30, font = 3, adj=0)

plot(res.spd[[1]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#BA9BC9")
plot(t.spd[[1]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[1]]$Phase[1])

plot(res.spd[[6]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#BA9BC9")
plot(t.spd[[6]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[6]]$Phase[1])

plot(res.spd[[8]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#BA9BC9")
plot(t.spd[[8]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[8]]$Phase[1])

plot(res.spd[[2]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#BA9BC9")
plot(t.spd[[2]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[2]]$Phase[1])

plot(res.spd[[5]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#BA9BC9")
plot(t.spd[[5]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[5]]$Phase[1])

plot(res.spd[[10]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#F6A83A")
plot(t.spd[[10]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[10]]$Phase[1])

plot(res.spd[[4]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#F6A83A")
plot(t.spd[[4]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[4]]$Phase[1])

plot(res.spd[[9]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#F6A83A")
plot(t.spd[[9]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[9]]$Phase[1])

plot(res.spd[[3]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#F6A83A")
plot(t.spd[[3]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[3]]$Phase[1])

plot(res.spd[[7]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#F6A83A")
plot(t.spd[[7]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[7]]$Phase[1])

plot(res.spd[[11]], ylim=c(0,0.005), calendar = "BCAD", fill.p = "#F6A83A")
plot(t.spd[[11]], type="simple", ylim=c(0,0.005), add=TRUE, lty=2, calendar = "BCAD")
mtext(dtbl[[11]]$Phase[1])

dev.off()

# Figure 2 - Permutation testing

par(mfrow=c(2,6))
par(mar=c(2,2,2,2))

plot.new()

plot(ptestn, focalm = "Arauquinoid", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Arauquinoid = 0.3447", font=1)

plot(ptestn, focalm = "Marajoara", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Marajoara = 0.1022", font=1)

plot(ptestn, focalm = "Paredo", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Paredo = 0.2754", font=1)

plot(ptestn, focalm = "Geoglyph", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Geoglyph = 0.0001", font=2)

plot(ptestn, focalm = "Loma", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Loma = 0.0333", font=2)


plot(ptestn, focalm = "Xinguano", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Xinguano = 0.4375", font=1)

plot(ptestn, focalm = "Koriabo", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Koriabo = 0.0043", font=2)

plot(ptestn, focalm = "Santarm", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Santarm = 0.0093", font=2)

plot(ptestn, focalm = "Guarita", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Guarita = 0.0783", font=1)

plot(ptestn, focalm = "Mound village", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Mound village = 1", font=1)

plot(ptestn, focalm = "Zanja", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Zanja = 0.2183", font=1)

dev.off()

# Figure 3 - Kernel density estimate comparisons

par(mfrow=c(3,2))
par(mar=c(2,2,2,2))

plot(res.kde[[1]], fill.col = rgb(0.73,0.6,0.78, 0.5), 
     line.col = rgb(0.73,0.6,0.78), line.type = 1, ylim=c(0, 0.010), calendar = "BCAD")
par(new=TRUE)
plot(res.kde[[4]], fill.col = rgb(0.96,0.65,0.22, 0.5),
     line.col = rgb(0.96,0.65,0.22), line.type = 1, ylim=c(0,0.010), calendar = "BCAD")
abline(v=1300, lty=2, lwd=2, col="darkgrey")
mtext("Guianas Coast", font=2)
text(labels = "Arauquinoid", y=0.0065, x=800, font = 3, adj=0, col = rgb(0.73,0.6,0.78))
text(labels = "Koriabo", y=0.0095, x=1500, font = 3, adj=0, col = rgb(0.96,0.65,0.22))

plot(res.kde[[6]], fill.col = rgb(0.73,0.6,0.78, 0.5),
     line.col = rgb(0.73,0.6,0.78), line.type = 1, ylim=c(0, 0.010), calendar = "BCAD")
par(new=TRUE)
plot(res.kde[[9]], fill.col = rgb(0.96,0.65,0.22, 0.5),
     line.col = rgb(0.96,0.65,0.22), line.type = 1, ylim=c(0,0.010), calendar = "BCAD")
abline(v=1100, lty=2, lwd=2, col="darkgrey")
mtext("Eastern Amazon", font=2)
text(labels = "Marajoara", y=0.005, x=700, font = 3, adj=0, col = rgb(0.73,0.6,0.78))
text(labels = "Santarm", y=0.006, x=1300, font = 3, adj=0, col = rgb(0.96,0.65,0.22))

plot(res.kde[[8]], fill.col = rgb(0.73,0.6,0.78, 0.5),
     line.col = rgb(0.73,0.6,0.78), line.type = 1, ylim=c(0, 0.010), calendar = "BCAD")
par(new=TRUE)
plot(res.kde[[3]], fill.col = rgb(0.96,0.65,0.22, 0.5),
     line.col = rgb(0.96,0.65,0.22), line.type = 1, ylim=c(0,0.010), calendar = "BCAD")
abline(v=1000, lty=2, lwd=2, col="darkgrey")
mtext("Central Amazon", font=2)
text(labels = "Paredo", y=0.0065, x=700, font = 3, adj=0, col = rgb(0.73,0.6,0.78))
text(labels = "Guarita", y=0.0065, x=1300, font = 3, adj=0, col = rgb(0.96,0.65,0.22))

plot(res.kde[[2]], fill.col = rgb(0.73,0.6,0.78, 0.5),
     line.col = rgb(0.73,0.6,0.78), line.type = 1, ylim=c(0, 0.010), calendar = "BCAD")
par(new=TRUE)
plot(res.kde[[7]], fill.col = rgb(0.96,0.65,0.22, 0.5),
     line.col = rgb(0.96,0.65,0.22), line.type = 1, ylim=c(0,0.010), calendar = "BCAD")
abline(v=940, lty=2, lwd=2, col="darkgrey")
mtext("Southwestern Amazon", font=2)
text(labels = "Geometric\nearthworks", y=0.004, x=500, font = 3, adj=0, col = rgb(0.73,0.6,0.78))
text(labels = "Ring villages", y=0.009, x=1000, font = 3, adj=0, col = rgb(0.96,0.65,0.22))

plot(res.kde[[5]], fill.col = rgb(0.73,0.6,0.78, 0.5),
     line.col = rgb(0.73,0.6,0.78), line.type = 1, ylim=c(0, 0.010), calendar = "BCAD")
par(new=TRUE)
plot(res.kde[[11]], fill.col = rgb(0.96,0.65,0.22, 0.5),
     line.col = rgb(0.96,0.65,0.22), line.type = 1, ylim=c(0,0.010), calendar = "BCAD")
abline(v=1300, lty=2, lwd=2, col="darkgrey")
mtext("Llanos de Moxos", font=2)
text(labels = "Lomas", y=0.004, x=1000, font = 3, adj=0, col = rgb(0.73,0.6,0.78))
text(labels = "Zanjas", y=0.005, x=1500, font = 3, adj=0, col = rgb(0.96,0.65,0.22))

plot(res.kde[[10]], fill.col = rgb(0.96,0.65,0.22, 0.5),
     line.col = rgb(0.96,0.65,0.22), line.type = 1, ylim=c(0, 0.010), calendar = "BCAD")
abline(v=1100, lty=2, lwd=2, col="darkgrey")
mtext("Southern Amazon", font=2)
text(labels = "Xinguano", y=0.007, x=1300, font = 3, adj=0, col = rgb(0.96,0.65,0.22))

dev.off()

## 5. SUPPLEMENTARY ANALYSIS

# 5.1 Custom mixing function for IntCal13 and SHCal13
# As of 30/08/2019 this extension of mixCurves() has been committed to the dev version of rcarbon (v1.3)

customMix <- function (calCurve1 = "intcal13", calCurve2 = "intcal13", p = 1, resOffsets = 0, 
                       resErrors = 0) 
{
  File1 <- paste(system.file("extdata", package = "rcarbon"), 
                 "/", calCurve1, ".14c", sep = "")
  File2 <- paste(system.file("extdata", package = "rcarbon"), 
                 "/", calCurve2, ".14c", sep = "")
  options(warn = -1)
  nh <- readLines(File1, encoding = "UTF-8")
  sh <- readLines(File2, encoding = "UTF-8")
  nh <- nh[!grepl("[#]", nh)]
  sh <- sh[!grepl("[#]", sh)]
  nh.con <- textConnection(nh)
  sh.con <- textConnection(sh)
  nh <- as.matrix(read.csv(nh.con, header = FALSE, 
                           stringsAsFactors = FALSE))[, 1:3]
  sh <- as.matrix(read.csv(sh.con, header = FALSE, 
                           stringsAsFactors = FALSE))[, 1:3]
  close(nh.con)
  close(sh.con)
  options(warn = 0)
  colnames(sh) <- c("CALBP", "C14BP", "Error")
  colnames(nh) <- c("CALBP", "C14BP", 
                    "Error")
  sh.mu <- approx(sh[, 1], sh[, 2], nh[, 
                                       1], rule = 2)$y + resOffsets
  sh.error <- approx(sh[, 1], sh[, 3], nh[, 
                                          1], rule = 2)$y
  sh.error <- sqrt(sh.error^2 + resErrors^2)
  mu <- p * nh[, 2] + (1 - p) * sh.mu
  error <- p * nh[, 3] + (1 - p) * sh.error
  res = cbind(nh[, 1], mu, error)
  colnames(res) = c("CALBP", "C14BP", "Error")
  return(res)
}

amazoncurve <- customMix(calCurve1 = "intcal13", calCurve2 = "shcal13", p = 0.5, resOffsets = 0, resErrors = 0)

# 5.2 Permutation test on truncated data

ptesttrunc <- permTest(calTrunc, marks = as.character(trunc$Phase), timeRange = timerange, nsim = 10000,
                   datenormalised = FALSE, spdnormalised = TRUE, bins = Tbins, runm=100)

# NOT RUN
# round(ptesttrunc$pValueList, 4)
#Koriabo*  Arauquinoid      Santarm*   Marajoara       Guarita       Paredo      Geoglyph*  Mound village       Zanja 
#0.0020        0.3626        0.0001        0.1942        0.2823        0.0564        0.0001        0.4604        0.1196 
#Loma*      Xinguano 
#0.0162        1.0000 

par(mfrow=c(2,6))
par(mar=c(2,2,2,2))

plot(ptesttrunc, focalm = "Arauquinoid", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Arauquinoid = 0.3626", font=1)

plot(ptesttrunc, focalm = "Marajoara", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Marajoara = 0.1942", font=1)

plot(ptesttrunc, focalm = "Paredo", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Paredo = 0.0564", font=1)

plot(ptesttrunc, focalm = "Geoglyph", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Geoglyph = 0.0001", font=2)

plot(ptesttrunc, focalm = "Loma", calendar = "BCAD", col.obs="#BA9BC9", lwd.obs = 2)
mtext("Loma = 0.0162", font=2)

plot.new()

plot(ptesttrunc, focalm = "Koriabo", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Koriabo = 0.002", font=2)

plot(ptesttrunc, focalm = "Santarm", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Santarm = 0.0001", font=2)

plot(ptesttrunc, focalm = "Guarita", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Guarita = 0.2823", font=1)

plot(ptesttrunc, focalm = "Mound village", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Mound village = 0.4604", font=1)

plot(ptesttrunc, focalm = "Zanja", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Zanja = 0.1196", font=1)

plot(ptesttrunc, focalm = "Xinguano", calendar = "BCAD", col.obs="#F6A83A", lwd.obs = 2)
mtext("Xinguano = 1", font=1)

